/*

This is the implementation file for the MyEventSegment
class.  This class defines funtions that can be used to 
readout any module covered in the CAENcard class.  These
include the V785, V775, and V792


Tim Hoagland
11/3/04
s04.thoagland@wittenberg.edu

*/




#include "MyEventSegment.h"              

static char* pPacketVersion = "1.0";           
// Packet version -should be changed whenever major changes are made
int Id=12;


//constructor set Packet details
MyEventSegment::MyEventSegment(unsigned int slot,  unsigned int crate, unsigned long  base):    
  m_MyPacket(Id, string("My Packet"), string("Sample documented packet"),
	     string(pPacketVersion))
{
  m_TDC = new CCAENV1x90(slot, crate, base);   
        // Creates a new CAENcard object looking at the indicated VME slot
}


// Is called right after the module is created.  All one time Setup
// should be done now.
void MyEventSegment::Initialize()
{
      // Set the trigger matching mode and associated parameters:

   m_TDC->TriggerMatchMode();         // (1)
   m_TDC->SetWindowWidth(3*40);       // (2)
   m_TDC->SetWindowOffset(-2*40);     // (3)
   m_TDC->SetExtraSearchMargin(1);    // (4)
   m_TDC->SetRejectMargin(1);         // (5)
   m_TDC->EnableTriggerTimeSubtraction(); // (6)

      // Set detection and resolution stuff:

   m_TDC->SetEdgeDetectMode(CCAENV1x90::EdgeMode_Leading); // (7)
   m_TDC->SetIndividualLSB(CCAENV1x90::Res_100ps);         // (8)
   m_TDC->SetDoubleHitResolution(CCAENV1x90::DT_5ns);      // (9)

     // Manage the event format:

   m_TDC->EnableTDCEncapsulation();                       // (10)
   m_TDC->SetMaxHitsPerEvent(CCAENV1x90::HITS_UNLIMITED); // (11)
   m_TDC->EnableErrorMark();                              // (12)
   m_TDC->SetErrorEnables(CCAENV1x90::ERR_VERNIER     |
                            CCAENV1x90::ERR_COARSE      |
                            CCAENV1x90::ERR_SELECT      |
                            CCAENV1x90::ERR_L1PARITY    |
                            CCAENV1x90::ERR_TFIFOPARITY |
                            CCAENV1x90::ERR_MATCHERROR  |
                            CCAENV1x90::ERR_RFIFOPARITY |
                            CCAENV1x90::ERR_RDOSTATE    |
                            CCAENV1x90::ERR_SUPPARITY   |
                            CCAENV1x90::ERR_CTLPARITY   |
                            CCAENV1x90::ERR_JTAGPARITY);   // (13)
   m_TDC->EnableAllChannels();                           // (14)
}

// Is called after reading data buffer
void MyEventSegment::Clear()
{
  m_TDC->Clear();          // Clear data buffer
}

unsigned int MyEventSegment::MaxSize()
{
  return 32*4+2;                
}


#define TDC_WAITLOOP 5                                   // (1)
static unsigned long TDCData[32768];

//Is called to readout data on module
DAQWordBufferPtr& MyEventSegment::Read(DAQWordBufferPtr& rBuf)
{
  for(int i =0; i < TDC_WAITLOOP; i++ ) {
     if(m_TDC->isEventFIFOReady()) {                      // (3)
        break;
     }
  }
  if(m_TDC->isEventFIFOReady()) {                         // (4)
     unsigned long fifo = m_TDC->ReadEventFIFO();         // (5)
     unsigned nWords    = m_TDC->FIFOWordCount(fifo);     // (6)
     
     unsigned nRead     = m_TDC->ReadData(TDCData, nWords); // (7)
     rBuf.CopyIn(TDCData, 0, nRead*2);                // (8)
     rBuf += nRead*2;                                // (9)
  }

  return rBuf;
}
